# Provable randomized rounding for minimum-similarity diversification

The repository for the "Provable randomized rounding for minimum-similarity diversification" paper

## Required Software

1. [pyltr](https://github.com/jma127/pyltr)
2. [MOSEK](https://www.mosek.com/downloads/)
    - This is the Quadratic Program solver that we use in the implementation
    - Please make sure you have the license and files set up
    - Alternatively, modify the `solve_QP` function in [minsumsim.py](./algorithms/minsumsim.py) with a QP solver syntax of your choice 

## Data

Refer to [DATASETS_README.md](./data/README.md) to get started with downloading and processing the required datasets. Then proceed to running experiments.

## Algorithms

To see the implementations of the algorithms go to [ALGORITHMS_README.md](./algorithms/README.md)

## Experiments

The experiments are described in [EXPERIMENTS_README.md](./experiments/README.md). 
Each experiment has its own configuration file which describes the particular algorithms that can be used and the dataset. For example, [lambda_0_alg_config.json](./experiments/lambda_0_alg_config.json) describes the algorithms and number of times to repeat each algorithm. Similarly, [lambda_0_data_config.json](./experiments/lambda_0_data_config.json) describes the locations of the datasets and properties such as if they are LETOR style data or if they are sparse.
To get a list of all the allowed algorithms and datasets use the `-h` option with the corresponding script.

Each experiment has its own associated `run_exp.py` file to execute the algorithms with the dataset and store the results in the [results_dir](./data/results). 

Each script has a `--overwrite_mode` argument which has options `{None, overwrite, append}`. Use this option if you wish to overwrite ior append to existing result files. 

### Run $\lambda =0$ experiments

To run the `minsumsim` algorithm on the `mq2008` dataset and overwrite the results folder use:

```
python run_lambda_0_exp.py minsumsim mq2008 --overwrite_mode overwrite
```

### Run $\lambda$ query experiments 

To collect the results of the `edge_greedy` algorithm on 100 queries of the `IstellaV` dataset for $\lambda=\{0.1,0.25,0.5,0.75,1\}$ and overwriting the results run the script:

```
python run_lambda_query_exp.py edge_greedy istella_vali --lambdas 0.1 0.25 0.5 0.75 1 --overwrite_mode overwrite
```

The results will be found in [file](./data/results/lambda_query/istella_vali/edge_greedy.csv)

### Run Subsampling experiments 

To run the subsampling algorithm for `minsumsim` algorithm on the `Census` dataset for 10 trials with 640 samples and append to results file:

```
python run_subsampling_exp.py minsumsim census --repetitions 10 --n_samples 640 --overwrite_mode append
```

The results will be present in [file](./data/results/subsampling/census/minsumsim.csv)

### Run Adversarial experiments

To gather the results for `node_greedy` algorithm for the adversarial example of size 100 with 20 "bad" items with $\lambda = \{0.1,1\}$, use:

```
python run_adversarial_greedy_exp.py node_greedy --N 100 --bad_examples 20 --lambdas 0.1 1 --overwrite_mode overwrite
```

The results can be seen under the folder `N_100_bad_20` in [results directory](./data/results/adversarial_greedy/)

## Results and Plots

All the code for the plots generated in the paper can be found in the [Jupyter notebooks](./notebooks/) 
