# Algorithms

## Shared python module
As the datasets gets larger, parallel processing on python incurs its own overhead.
Due to the process based palletization, objects are pickled and sent to sub-processes.
This can cause a blow up in memory when using the larger datasets.
Hence, to share the loaded dataset without passing it as a parameter, we store it in a common module named [shared.py](./shared.py).
By importing this module we use the shared memory object which is already loaded.

All algorithms are written in a way that an explicit data matrix can instead be sent to allow for flexible testing where memory is not a big concern.

## MinSumSim Algorithm

Minimizes the cost of the objective below
$$   cost(S) = \lambda\sum_{\vec x \in S}\rho({\vec x}) 
          + \sum_{\vec x\in S}\sum_{\substack{\vec y \in S \\ \vec y \neq \vec x}} \sigma({\vec x},{\vec y})
$$

where $\rho:X\rightarrow \mathbb{R}_+$ is a _relevance loss_ function and 
$\sigma: X\times X\rightarrow [0,1]$ is a _similarity_ function.

Uses MOSEK to solve the Quadratic Problem to get result vector $\mathbf{z}$. Then randomized rounding is performed to get discrete solution $\mathbf{x}$. This solution is not guaranteed to be feasible, therefore we run randomized rounding multiple times till we get a feasible solution. After gathering many feasible solution using default values of $\epsilon=0.01$ and $\delta=0.01$. Then we select the best feasible solution as the final solution.

See [minsumsim.py](./minsumsim.py) for implementation.

## Node Greedy Algorithm

Adapts the greedy algorithm proposed by Borodin et al\. for the objective function mentioned above.
The implementation converts the problem to minimization of similarities and runs the greedy algorithm. See implementation in [node_greedy.py](./node_greedy.py)

## Edge Greedy Algorithm

Adapts the greedy algorithm proposed by Gollapudi et al\. to select edges first that minimize the objective at hand. This algorithm requires the similarity matrix and runs in $\mathcal{O}(n^2)$ and is therefore slow or memory intensive for any reasonably sized dataset. See implementation in [edge_greedy.py](./edge_greedy.py)
