from time import time
import numpy as np
import algorithms.shared as shared
from algorithms.util import find_obj

def random_vector_baseline(k, initial_states=1000):
    sols = []
    A = shared.A
    N = A.shape[0]
    for _ in range(initial_states):
        r = np.random.choice(N, k, replace=False)
        i = np.zeros(N)
        i[r] = 1
        sols.append(i)

    return sols


def get_random_stats(k, initial_states =1000,time_lim=None):

    possible_sols = []
    start = time()
    r = random_vector_baseline(k, initial_states=1)
    run_time = time() - start

    possible_sols.extend(r)

    if time_lim is None:
        # repeat 1000-1 times
        repeat = initial_states-1
    else:
        rem = time_lim - run_time
        repeat = int(np.ceil(rem / run_time))

    start = time()
    r = random_vector_baseline(k, initial_states=repeat)
    loop_time = time() - start

    possible_sols.extend(r)
    start = time()
    obj = [find_obj(sol) for sol in possible_sols]
    # _,obj = find_obj_parallel(possible_sols,A)

    best_sol = min(obj)
    worst_sol = max(obj)
    avg_sol = np.mean(obj)

    sol_search_time = time() - start

    total_time = run_time + loop_time + sol_search_time
    stat = {
        "algo": "Random_baseline",
        "k": k,
        "min_obj": best_sol,
        "max_obj": worst_sol,
        "avg_obj": avg_sol,
        "single_run_time": run_time,
        "loop_time": loop_time,
        "sol_search_time": sol_search_time,
        "total_time": total_time,
        "initial_states":repeat+1
    }

    return stat