from itertools import combinations

import algorithms.shared as shared

from sklearn.preprocessing import normalize

import numpy as np

from functools import partial

import multiprocessing as mp

def preprocess_data(letor=False):
    A = shared.A
    # print("preprocess local A: ",id(A))
    if letor:
        rel = A[:, 0].astype(float)
        qid = A[:, 1].astype(str)
        A = A[:, 2:].astype(np.float)
        # normalize data matrix
        normalize(A, copy=False)
        # ignore 0 vector rows
        mask = A.sum(axis=1) == 0
        if sum(mask):
            A = A[~mask]
            rel = rel[~mask]
            qid = qid[~mask]
        return rel, qid, A

    else:
        normalize(A, copy=False)
        mask = A.sum(axis=1) == 0
        if sum(mask):
            A = A[~mask]
        return None,None,A


def find_obj(x, irrelevance=None,A=None):
    if A is None:
        A = shared.A
    # print("find obj local A: ",id(A))
    S = np.argwhere(x == 1).flatten()
    obj = 0
    for (u, v) in combinations(S, 2):
        obj += A[u] @ A[v]
    # double it to match x.T@W@x
    obj = 2 * obj
    if irrelevance is not None:
        obj += (x.T @ irrelevance)
    return obj


def find_opt_solution(k):
    N = shared.A.shape[0]
    if N > 50 or k > 7:
        print("Too many nodes or k to compute optimal")
        return
    min_obj = np.infty
    best = None
    for S in combinations(range(N), k):
        x = np.zeros(N)
        x[list(S)] = 1
        temp = find_obj(x)
        if temp < min_obj:
            min_obj = temp
            best = S
    return best, min_obj


# a function that takes in a dummy input and function and just executes that function
# This is needed as parallel code requires a function that takes in an input
def f(_, func):
    return func()


# a wrapper that takes in a function and return a function with an extra input
def wrapper_parallel(func):
    return partial(f, func=func)


class NoDaemonProcess(mp.Process):
    # make 'daemon' attribute always return False
    def _get_daemon(self):
        return False
    def _set_daemon(self, value):
        pass
    daemon = property(_get_daemon, _set_daemon)

# We sub-class multiprocessing.pool.Pool instead of multiprocessing.Pool
# because the latter is only a wrapper function, not a proper class.
class MyPool(mp.pool.Pool):
    Process = NoDaemonProcess

class NoDaemonProcess(mp.Process):
    @property
    def daemon(self):
        return False

    @daemon.setter
    def daemon(self, value):
        pass


class NoDaemonContext(type(mp.get_context())):
    Process = NoDaemonProcess

# We sub-class multiprocessing.pool.Pool instead of multiprocessing.Pool
# because the latter is only a wrapper function, not a proper class.
class NestablePool(mp.pool.Pool):
    def __init__(self, *args, **kwargs):
        kwargs['context'] = NoDaemonContext()
        super(NestablePool, self).__init__(*args, **kwargs)