from preprocess.process_letor import LETOR_datasets, create_leotor_npz
from preprocess.process_istella import istella_to_min_npz
from preprocess.process_census import census_to_npz
from pathlib import Path
import argparse


project_root = Path(__file__).resolve().parent


def get_args():
    parser = argparse.ArgumentParser(
        description="Preprocesses datasets and saves as npz file"
    )

    parser.add_argument(
        "dataset",
        help="Which dataset to process",
        choices=["all","letor", "istella_vali", "istella_test", "census"],
    )
    parser.add_argument(
        "--raw_data_dir",
        type=Path,
        help="Location of raw data directory",
        default=project_root / "data" / "raw",
    )

    parser.add_argument(
        "--save_dir",
        type=Path,
        help="Location to save npz files",
        default=project_root / "data" / "processed",
    )
    args = parser.parse_args()
    return args


if __name__ == "__main__":
    args = get_args()
    if not args.raw_data_dir.exists():
        raise FileNotFoundError(
            f"Raw data folder: {args.raw_data_dir} doesn't exist, please download and save datasets in raw folder"
        )

    if not args.save_dir.exists():
        args.save_dir.mkdir()

    if args.dataset == "all":
        create_leotor_npz(LETOR_datasets, args.raw_data_dir, args.save_dir)
        istella_to_min_npz("vali", args.raw_data_dir, args.save_dir)
        istella_to_min_npz("test", args.raw_data_dir, args.save_dir)
        census_to_npz(args.raw_data_dir,args.save_dir)
    
    elif args.dataset == "letor":
        create_leotor_npz(LETOR_datasets, args.raw_data_dir, args.save_dir)
    
    elif args.dataset == "istella_vali":
        istella_to_min_npz("vali", args.raw_data_dir, args.save_dir)
    
    elif args.dataset == "istella_test":
        istella_to_min_npz("test", args.raw_data_dir, args.save_dir)
    
    elif args.dataset == "census":
        census_to_npz(args.raw_data_dir,args.save_dir)
        
