# Experiments

The various experiments described in the paper are implemented here.
Each experiment has an associated algorithm (and possibly a data) configuration file.
This indicates the location of the algorithms that can be used in the experiments and can be modified to try out different combinations.

## $\lambda=0$ Experiments

As most of the datasets have relevance scores related to a query, we cannot use the complete datasets along with relevance scores to find $k$ most diverse items.
Therefore, we set the tradeoff parameter $\lambda$ to $0$, indicating that we only care about minimizing the sum of pairwise distances between the objects.
This allows us to run experiments with all the datasets as a whole to test out scalability.

The [lambda_0.py](./lambda_0.py) has a function that essentially loads the correct dataset and algorithms. Then runs the algorithm for the range of $k$s and collects the results.

__A point to note__ : As the `edge_greedy` algorithm is qudractic in space and running time it is infeasible to run it for any dataset other than the smallest dataset, i.e., `mq2008`

## $\lambda$-Query Experiments

Now, to study the effects of the tradeoff parameter $\lambda$ we run the algorithms only for the related query and the relevance scores provided in the letor style datasets.

The `mq2008` and `mq2007` datasets have on average 40 documents associated with each query, the `gov_2003_hp` dataset has on average $100$ documents per query. The `istella_vali` dataset has $5000$ documents per query. Furthermore, the `LETOR` datasets have only relevance scores $\{0,1,2\}$ while, the `Istella-X` datasets have a richer relevance scores of $\{0,1,2,3,4\}$. Therefore, we primarily run the experiments for hte `istella_vali` dataset.

As all the algorithms are minimizing an objective we, need to covert the relevance scores into _relevance losses_.
This is refered to as __irrelevance scores__ in parts of the implementations.
First, we convert the relevance scores into a range $r \in (0,1]$.
Then, we compute the relevance scores as $\rho(x) = 1+\log(1/r)$.

The [lambda_query.py](./lambda_query.py) has a function that loads the dataset and the algorithm.
Then it collects the query ids of the largest queries.
It proceeds to run the algorithm for $100$ largest queries and stores the results.

## Adversarial Example Experiments

This experiment aims to create an adversarial example that will cause the greedy algorithms to perform poorly by picking "_bad_" documents  initially that will cause it end up with a larger objective.

Assume that data matrix $X$ is partitioned as $X= Y\cup Z$. Documents in $Y$ are mostly uncorrelated with each other, as are documents in $Z$. The setting can be achieved with a matrix with the following structure, where $a\gg b$:
$$
A=\left ( \begin{array}{ccccccc}
  a & b & b &   a & b & b & 
\\  b & a & b &  b & a & b &
\\ b & b & a &   b & b & a &  \cdots
\\ a & a & a &   b & b & b &
\\   b & b & b & a & a & a & 
\\    &  &  &\vdots  &  &  & 
\end{array} \right ).
$$

If the greedy algorithm begins with a document in $Y$, maybe due to a marginally higher relevance score, then the overall selection of the other $k-1$ documents is suboptimal.

We generate this bad example from a gamma distribution and skew the relevance scores slightly such that a "bad" document is picked by the greedy algorithms.
The experiments in [adversarial_greedy.py](adversarial_greedy.py) generate multiple instances of an adversarial example and collect results for the chosen algorithm.

## Subsampling Experiments

Although, the `minsumsim` algorithm scales to large datasets, in time-sensitive cases, the greedy algorithm might seem more ideal.

Given an instance $(X, k, \lambda)$ of size $n$ we sample a small number of items (say $\sqrt n$) from $X$, uniformly without replacement, to produce a new instance $(\tilde X, k, \lambda)$, which can be solved much faster. We sample and solve a large number of such instances, which can be done in a distributed fashion, and take the union of the solutions, each of size $k$, to produce a final instance $(\hat X, k, \lambda)$, which we solve to produce the final output.

This methods leads to significant speedup when using large datasets making it more comparable with the greedy algorithms.
The implementation is seen in [subsampling.py](subsampling.py).