#%%
import numpy as np
from algorithms.util import find_obj, preprocess_data
import algorithms.shared as shared
from experiments.utils import import_obj
from pathlib import Path
from tqdm import tqdm
from time import time

project_root = Path(__file__).resolve().parent.parent

#%%
def new_wrecker(good_num, bad_num):
    s1 = bad_num
    s2 = good_num
    big = 1000
    small = 1
    N = s1 * s2
    A1 = np.hstack([np.eye(s1) * (big - small) + small] * s2)
    base2 = np.hstack([np.ones(s1) * big, np.ones(N - s1) * small])
    A2 = np.vstack([np.roll(base2, s1 * i) for i in range(int(s2))])
    A = np.vstack([A1, A2])
    return np.random.gamma(A, scale=0.1, size=A.shape)


def gen_relevance_scores(N):
    # rel = 1+ (np.random.random(N)-0.5)*0.1
    rel = np.repeat(0.1, N)
    rel[0] = np.max(rel) + 1e-3
    return rel


def load_wrecking_data(N, bad_examples):
    good_examples = int(N - bad_examples)

    A = new_wrecker(good_examples, bad_examples)
    shared.update_A(A)
    _, _, A = preprocess_data(letor=False)
    shared.update_A(A)
    # * important clear W after every new data matrix
    shared.update_W(None)
    return A


def dict_2_string(d):
    strings = []
    algo = d["algo"]
    strings.append(algo)
    strings.append(d["k"])
    strings.append(d["lam"])
    if algo == "MinSimSum":
        strings.append(d["min_obj"])
        strings.append(d["QP_time"])
        strings.append(d["rounding_time"])
        strings.append(d["sol_search_time"])

    elif algo == "Greedy_B_irr" or algo == "Greedy_A_irr":
        strings.append(d["min_obj"])
        strings.append(d["run_time"])
        strings.append(d["loop_time"])
        strings.append(d["obj_time"])
    else:
        print("Unknown algorithm please check code")
        raise UserWarning

    strings.append(d["total_time"])
    stat_string = ",".join([str(i) for i in strings])
    stat_string += "\n"
    return stat_string


def execute_adversarial_greedy(args, alg_dict, overwrite_mode="None"):

    ks = range(3, args.N)

    # location of the results folder
    results_folder = (
        project_root
        / "data"
        / "results"
        / "adversarial_greedy"
        / f"N_{args.N}_bad_{args.bad_examples}"
    )
    # create if it doesn't exist
    if not results_folder.exists():
        results_folder.mkdir(parents=True, exist_ok=True)

    results_file = results_folder / f"{alg_dict['name']}.csv"
    file_exists = results_file.exists()

    if overwrite_mode == "None" and file_exists:
        raise ValueError(
            f"Result file for adversarial example N:{args.N} and bad_examples:{args.bad_examples} and algorithm {alg_dict['name']} already exists. Change overwrite mode to overwrite or append to gather new results"
        )
    elif overwrite_mode == "overwrite":
        mode = "w"
        file_exists = False
    elif overwrite_mode == "append":
        mode = "a"
    else:
        mode = "a"

    # generate relevances scores
    relevances = gen_relevance_scores(args.N)
    # convert to irrelevance scores
    irrelevances = 1 + np.log(1 / relevances)

    # load the algorithm 
    stat_fn = import_obj(alg_dict['stat_fn'])

    with open(results_file, mode=mode, encoding="utf-8") as fp:
        if not file_exists:
            fp.write("algorithm,k,lam,min_obj,t_1,t_2,t_3,t_total\n")
        for _ in tqdm(range(args.repetitions)):
            # create an instance of adversarial example
            A = load_wrecking_data(args.N, args.bad_examples)
            for k in tqdm(ks):
                for lam in tqdm(args.lambdas):
                    stat = stat_fn(
                        k=k, irrelevance=irrelevances, lam=lam
                    )
                    stat_str = dict_2_string(stat)
                    fp.write(stat_str)
                    fp.flush()


# %%
