import pandas as pd
import numpy as np
from pathlib import Path


def census_to_npz(raw_data_dir, save_dir):
    print("Loading Census dataset")
    census_file = raw_data_dir / "USCensus1990.data.txt"
    if not census_file.exists():
        raise FileNotFoundError(
            f"Census dataset not found at {census_file}, please download and place in raw data directory"
        )

    df = pd.read_csv(census_file)
    df["vector"] = df.iloc[:, 2:].values.tolist()
    df.head()
    df["vector"] = df["vector"].apply(lambda x: np.array(x, dtype=np.float64))
    A = np.array(df.vector.tolist())

    print("Saving census npz. Might take a while...")

    save_file = save_dir / "census.npz"
    np.savez_compressed(save_file, census=A)

    print(f"File saved at {save_file}")

