#%%
import numpy as np
import pandas as pd
from pathlib import Path

import pyltr

# location of the LETOR datasets in the raw dataset folder
# We choose the Querylevel normalized version of each dataset 
#%%
LETOR_datasets = {
    'mq2007':'MQ2007/Querylevelnorm.txt',
    
    'mq2008': 'MQ2008/Querylevelnorm.txt',
    
    'gov_2003_hp':'Gov/QueryLevelNorm/2003_hp.txt',
    
    'gov_2003_np':'Gov/QueryLevelNorm/2003_np.txt',
    
    'gov_2003_td':'Gov/QueryLevelNorm/2003_td.txt',
    
    'gov_2004_hp':'Gov/QueryLevelNorm/2004_hp.txt',
    
    'gov_2004_np':'Gov/QueryLevelNorm/2004_np.txt',
      
    'gov_2004_td':'Gov/QueryLevelNorm/2004_td.txt',
        
}

def create_leotor_npz(dataset_locs,raw_data_dir,save_dir): 
    """Created compressed numpy file of all letor datasets.
    The first column is the relevance scores.
    The second is the query.
    The following columns are the features

    Args:
        dataset_locs (dict): contains the names and location of raw LETOR files
        raw_data_dir (Path): location of raw data folder
        save_dir (Path): location to save npz file 
    """
    kwargs ={}
    print("Loading LETOR datasets into numpy")
    for name,location in dataset_locs.items():
        with open(raw_data_dir/location,'r') as fp:
            # get the features, relevance scores, query ids and comments
            X,rel,qid,comments = pyltr.data.letor.read_dataset(fp)
        # create combined matrix 
        A = np.column_stack((rel,qid,X))
        kwargs[name] = A
    print("Saving LETOR to npz file. This might take a while....")
    save_file = save_dir/'LETOR_compressed.npz'
    np.savez_compressed(save_file,**kwargs)
    print(f"File saved at {save_file}")

