import json
import argparse
from experiments.subsampling import execute_subsampling


with open("experiments/subsampling_alg_config.json",'r') as fp:
    alg_dict = json.load(fp)
with open("experiments/lambda_0_data_config.json",'r') as fp:
    data_dict = json.load(fp)

def get_args():
    parser = argparse.ArgumentParser(
        description="Collect results for Subsampling experiment"
    )
    parser.add_argument(
        "algorithm",
        type=str,
        help="The algorithm to run",
        choices=alg_dict.keys(),
    )
    parser.add_argument(
        "dataset",
        type=str,
        help="The dataset to use",
        choices=data_dict.keys()
    )
    parser.add_argument("--repetitions", type=int, default=10, help="Times to repeat subsampling")
    parser.add_argument(
        "--n_samples",
        type=int,
        nargs="+",
        default=[10,20,40,80,160,320,640,1280],
        help="The number of samples to gather results for"
    )
    parser.add_argument(
        "--overwrite_mode",
        type=str,
        default="None",
        help="Whether to overwrite or append results",
        choices=["None", "overwrite", "append"],
    )

    args = parser.parse_args()
    return args

#%%
if __name__ == "__main__":
    args = get_args()
    execute_subsampling(args,data_dict[args.dataset],alg_dict=alg_dict[args.algorithm],overwrite_mode=args.overwrite_mode)